#include <Arduino.h>
#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <AsyncTCP.h>
#include <ESPAsyncWebServer.h>
#include "SPIFFS.h"
#include <Arduino_JSON.h>
#include <SparkFunHTU21D.h>
#include <Adafruit_Sensor.h>

#define LED 2
#define GAS_SENSOR_A 39
#define GAS_SENSOR_D 36
#define IR_SENSOR 34

int ir_var = 1;
int gassensorAnalog = 0;

const char* ssid = "DIGI-8779";
const char* password = "CdanaM2587";
const char* host = "maker.ifttt.com";
const char* apiKey = "f4i315_kOXujKT6W1SKEmU-kDSi50glvCeiBt6k7R1K";

// Create AsyncWebServer object on port 80
AsyncWebServer server(80);

// Create an Event Source on /events
AsyncEventSource events("/events");

// Json Variable to Hold Sensor Readings
JSONVar readings;

// Timer variables
unsigned long lastTime = 0;
unsigned long timerDelay = 10000;

int lock = 0, lock_gas = 0 ;

// Create a sensor object
HTU21D myHumidity; // BME280 connect to ESP32 I2C (GPIO 21 = SDA, GPIO 22 = SCL)

// Init HTU21D
void initHTU21D(){
    myHumidity.begin();
}

// Get Sensor Readings and return JSON object
String getSensorReadings(){
  readings["temperature"] = String(myHumidity.readTemperature());
  readings["humidity"] =  String(myHumidity.readHumidity());
  String jsonString = JSON.stringify(readings);
  return jsonString;
}

// Initialize SPIFFS
void initSPIFFS() {
  if (!SPIFFS.begin()) {
    Serial.println("An error has occurred while mounting SPIFFS");
  }
  Serial.println("SPIFFS mounted successfully");
}

// Initialize WiFi
void initWiFi() {
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.print("Connecting to WiFi ..");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print('.');
    delay(1000);
  }
  Serial.println(WiFi.localIP());
}

void setup() {
  // Serial port for debugging purposes
  Serial.begin(115200);
  initHTU21D();
  initWiFi();
  initSPIFFS();

  pinMode(LED, OUTPUT);
  pinMode(GAS_SENSOR_A, INPUT);
  pinMode(IR_SENSOR, INPUT);

  // Web Server Root URL
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send(SPIFFS, "/index.html", "text/html");
  });

  server.serveStatic("/", SPIFFS, "/");

  // Request for the latest sensor readings
  server.on("/readings", HTTP_GET, [](AsyncWebServerRequest *request){
    String json = getSensorReadings();
    request->send(200, "application/json", json);
    json = String();
  });

  events.onConnect([](AsyncEventSourceClient *client){
    if(client->lastId()){
      Serial.printf("Client reconnected! Last message ID that it got is: %u\n", client->lastId());
    }
    // send event with message "hello!", id current millis
    // and set reconnect delay to 1 second
    client->send("hello!", NULL, millis(), 10000);
  });
  server.addHandler(&events);

  // Start server
  server.begin();
}

void loop() {
  if ((millis() - lastTime) > timerDelay) {
    // Send Events to the client with the Sensor Readings Every 10 seconds
    events.send("ping",NULL,millis());
    events.send(getSensorReadings().c_str(),"new_readings" ,millis());
    digitalWrite(LED, !digitalRead(LED));
    lock = 0;
    lastTime = millis();
  }

  if(!lock)
  {
    ir_var = digitalRead(IR_SENSOR);
    if(ir_var == LOW)
    {
      lock = 1;
      Serial.print("Motion detect");
      Serial.print("connecting to ");
      Serial.println(host);
      WiFiClient client;
      const int httpsPort = 80;
      if (!client.connect(host, httpsPort)) {
        Serial.println("connection failed");
        return;
      }

      String url = "/trigger/motion_detect/with/key/";
      url += apiKey;

      Serial.print("Requesting URL: ");
      Serial.println(url);
      client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + host + "\r\n" +
               "User-Agent: BuildFailureDetectorESP32\r\n" +
               "Connection: close\r\n\r\n");

      Serial.println("request sent");
      while (client.connected()) {
        String line = client.readStringUntil('\n');
        if (line == "\r") {
          Serial.println("headers received");
          break;
        }
      }
      String line = client.readStringUntil('\n');
      Serial.println("reply was:");
      Serial.println("==========");
      Serial.println(line);
      Serial.println("==========");
      Serial.println("closing connection");
      
    }
    if(!lock_gas)
    {
      gassensorAnalog = analogRead(GAS_SENSOR_A);
      
      if(gassensorAnalog > 700)
      {
        lock_gas = 1;
        Serial.print("connecting to ");
      Serial.println(host);
      WiFiClient client;
      const int httpsPort = 80;
      if (!client.connect(host, httpsPort)) {
        Serial.println("connection failed");
        return;
      }

      String url = "/trigger/gas_detect/with/key/";
      url += apiKey;

      Serial.print("Requesting URL: ");
      Serial.println(url);
      client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + host + "\r\n" +
               "User-Agent: BuildFailureDetectorESP32\r\n" +
               "Connection: close\r\n\r\n");

      Serial.println("request sent");
      while (client.connected()) {
        String line = client.readStringUntil('\n');
        if (line == "\r") {
          Serial.println("headers received");
          break;
        }
      }
      String line = client.readStringUntil('\n');
      Serial.println("reply was:");
      Serial.println("==========");
      Serial.println(line);
      Serial.println("==========");
      Serial.println("closing connection");
      }
    }
    
  }
  
}
